<?php
/**
 * [PHPFOX_HEADER]
 */

defined('PHPFOX') or exit('NO DICE!');

/**
 * 
 * 
 * @copyright		[PHPFOX_COPYRIGHT]
 * @author  		Raymond Benc
 * @package 		Phpfox_Service
 * @version 		$Id: api.class.php 2900 2011-08-26 09:18:42Z Raymond_Benc $
 */
class Api_Service_Api extends Phpfox_Service 
{
	private $_aApi = array();
	private $_bError = false;
	private $_sFormat = 'xml';
	private $_aOutput = array();
	private $_iTotal = 0;
	/**
	 * Class constructor
	 */	
	public function __construct()
	{	
		$this->_sTable = Phpfox::getT('app');	
		if ($this->get('format') != '')
		{
			$this->_sFormat = $this->get('format');	
		}
	}

	public function getAppName()
	{
		return $this->_aApi['app_title'];
	}
	
	public function process()
	{
		$this->_aApi = $this->database()->select('*')
			->from($this->_sTable)
			->where('public_key = \'' . $this->database()->escape($this->get('public_key')) . '\'')
			->execute('getSlaveRow');		
		
		if (!isset($this->_aApi['app_id']))
		{
			$this->error('api.unable_to_find_api_key', 'Unable to find API key.');	
			return $this->_sendResponse();
		}
		
		if (isset($this->_aApi['private_key']) && $this->get('private_key') != $this->_aApi['private_key'])
		{
			$this->error('api.private_key_does_not_match', 'Private key does not match.');
			return $this->_sendResponse();
		}
		
		if ($this->get('method') == '')
		{
			$this->error('api.method_not_defined', 'Method not defined.');
			return $this->_sendResponse();
		}
		
		$aParts = explode('.', $this->get('method'));
		
		if (!isset($aParts[1]))
		{
			$this->error('api.method_not_correctly_defined', 'Method not correctly defined.');
			return $this->_sendResponse();
		}		
		
			
		$sModule = $aParts[0];
		$sMethod = $aParts[1];

		if (!Phpfox::isModule($sModule))
		{
			$this->error('5', 'Module does not exist.');
			return $this->_sendResponse();
		}

		/* the token links a user with viewing an app, an app should not query the site
		 * unless a user is viewing the app. 
		 */
		$sWhere = 'app_id = ' . $this->_aApi['app_id'] . 
				' AND token = "' . Phpfox::getLib('parse.input')->clean($this->get('token')) . '"'.
				' AND time_stamp >= ' . (PHPFOX_TIME - Phpfox::getParam('apps.token_keep_alive'));
		$aAccess = $this->database()->select('aa.access_id, aa.user_id, u.user_group_id')
				->from(Phpfox::getT('app_access'),'aa')
				->join(Phpfox::getT('user'), 'u', 'u.user_id = aa.user_id')
				->where($sWhere)
				->execute('getSlaveRow');
		
		if (empty($aAccess['access_id']))
		{
			$this->error('api.not_a_valid_token', 'Not a valid token.');
			
			return $this->_sendResponse();
		}
		
		$this->_aApi['target_user_id'] = $aAccess['user_id'];
		// Used to forge Phpfox::getUserId()		
		define('PHPFOX_APP_USER_ID', $aAccess['user_id']);
		// Used to skip passing params everywhere
		define('PHPFOX_APP_ID', $this->_aApi['app_id']);
		define('PHPFOX_APP_USER_GROUP_ID', $aAccess['user_group_id']);
		
		$oService = Phpfox::getService($sModule . '.api');
		if (!method_exists($oService, $sMethod))
		{
			$this->error('api.method_not_valid_for_module', 'Method for this module does not exist.');
			return $this->_sendResponse();
		}
				
		$mOutput = $oService->$sMethod();
		
		if ($this->isPassed())
		{
			$this->_aOutput = $mOutput;
			
			return $this->_sendResponse();
		}
		
		$this->_aOutput = array_merge(array('error' => $this->_bError), $this->_aOutput);			
		
		return $this->_sendResponse();
	}
	
	public function getUserId()
	{
		return (int) $this->_aApi['target_user_id'];
	}
	
	public function setTotal($iTotal)
	{
		$this->_iTotal = (int) $iTotal;
	}
	
	public function isAllowed($sVariable, $iUserId = null, $iAppId = null)
	{
		static $aPrivileges;
		
		if ($iAppId === null)
		{
			$iAppId = $this->_aApi['app_id'];
		}
		
		if ($iUserId === null)
		{
			$iUserId = $this->_aApi['target_user_id'];
		}
		
		if (!isset($aPrivileges[$iAppId]) && !isset($aPrivileges[$iAppId][$iUserId]))
		{
			$aRows = $this->database()->select('var_name')
					->from(Phpfox::getT('app_disallow'))
					->where('app_id = ' . (int)$iAppId . ' AND user_id = ' . (int)$iUserId . '')
					->execute('getSlaveRows');
			
			$aPrivileges[$iAppId][$iUserId] = array();
			foreach ($aRows as $aRow)
			{
				$aPrivileges[$iAppId][$iUserId][$aRow['var_name']] = true;
			}
		}
		
		return (isset($aPrivileges[$iAppId][$iUserId][$sVariable]) ? false : true);
	}
	
	public function get($sRequest)
	{
		return Phpfox::getLib('request')->get($sRequest);
	}	
	
	public function isPassed()
	{
		return ($this->_bError ? false : true);
	}
	
	public function error($iErrorId, $sErrorMessage)
	{
		$this->_bError = true;	
		$this->_aOutput = array(
			'error_id' => $iErrorId,
			'error_message' => $sErrorMessage
		);
		
		return false;
	}	
	
	private function _sendResponse()
	{		
		$sOutput = json_encode(array(
				'api' => array(
					'total' => $this->_iTotal,
					'pages' => Phpfox::getLib('pager')->getTotalPages(),
					'current_page' => Phpfox::getLib('pager')->getCurrentPage()
				),
				'output' => $this->_aOutput
			)
		);	
		echo $sOutput;		
	}
	
	/**
	 * If a call is made to an unknown method attempt to connect
	 * it to a specific plug-in with the same name thus allowing 
	 * plug-in developers the ability to extend classes.
	 *
	 * @param string $sMethod is the name of the method
	 * @param array $aArguments is the array of arguments of being passed
	 */
	public function __call($sMethod, $aArguments)
	{
		/**
		 * Check if such a plug-in exists and if it does call it.
		 */
		if ($sPlugin = Phpfox_Plugin::get('api.service_api__call'))
		{
			return eval($sPlugin);
		}
			
		/**
		 * No method or plug-in found we must throw a error.
		 */
		Phpfox_Error::trigger('Call to undefined method ' . __CLASS__ . '::' . $sMethod . '()', E_USER_ERROR);
	}	
}

?>